const invariant = require('invariant');
const semver = require('semver');

module.exports = function checkDependencyVersions() {
  // get dependency versions from project root (not local node_modules)
  const webpackVersion = require.main.require('webpack/package').version;
  const babelLoaderVersion = require.main.require('babel-loader/package')
    .version;
  const babelCoreVersion = require.main.require('babel-core/package').version;
  const pkgJson = require('./package');
  const minWebpackVersion = pkgJson.peerDependencies['webpack'];
  const minBabelLoaderVersion = pkgJson.dependencies['babel-loader'];
  const minBabelCoreVersion = pkgJson.dependencies['babel-core'];

  // make sure user hasn't installed unsupported versions of crucial packages
  [
    ['webpack', minWebpackVersion, webpackVersion],
    ['babel-core', minBabelCoreVersion, babelCoreVersion],
    ['babel-loader', minBabelLoaderVersion, babelLoaderVersion],
  ].forEach(([p, m, v]) =>
    invariant(
      v && m && semver.satisfies(v, m),
      'wub is designed to work with %s@%s. %s %s is not supported.',
      p,
      m,
      p,
      v
    )
  );
};
