const webpack = require('webpack');
const path = require('path');
const projectDir = path.dirname(module.parent.filename);

module.exports = (env, webpackOptions) => ({
  output: {
    path: path.join(projectDir, 'build'),
    pathinfo: true,
    publicPath: '/',
    filename: 'bundle.[name].[hash].js',
  },
  plugins: [
    // enable named modules in development
    webpackOptions.hot && new webpack.NamedModulesPlugin(),
    new webpack.ProgressPlugin(),
    new webpack.NoEmitOnErrorsPlugin(),
  ].filter(f => f),
  devServer: {
    hot: true,
    historyApiFallback: true,
    // show an error overlay when there's a compilation issue
    overlay: true,
    compress: true,
    clientLogLevel: 'none',
  },
  performance: webpackOptions.hot ? { hints: false } : {},
  module: {
    // error if module doesn't export anything
    strictExportPresence: true,
    rules: [
      {
        test: /\.jsx?$/,
        exclude: modulePath =>
          // it's in a node_modules folder
          modulePath.indexOf('node_modules') > -1 &&
          // and it's not a file required by a preset
          !/wub-preset-\w+\/[^/]+$/.test(modulePath),
        use: [
          {
            loader: require.resolve('babel-loader'),
            options: {
              presets: [
                [
                  'env',
                  {
                    targets: {
                      browsers: [
                        '>1%',
                        'last 4 versions',
                        'not ie < 9',
                        'firefox esr',
                      ],
                    },
                    modules: false,
                  },
                ],
              ],
              plugins: [
                'transform-object-rest-spread',
                'transform-class-properties',
                'syntax-dynamic-import',
              ],
            },
          },
        ],
      },
    ],
  },
});
