const invariant = require('invariant');
const path = require('path');
const webpackMerge = require('webpack-merge');

const checkDeps = require('./checkDependencyVersions');

const normalisePresets = require('./normalisePresets');
const { styleEnd, styleStart } = require('./utils/console-format');

// Delete the module from the require cache so it gets loaded fresh every time.
// module.parent.filename always refers to the first parent that loaded wub
// unless the cache is cleared. This is mainly for webpack multi-compiler support.
delete require.cache[__filename];

module.exports = presets => (env, webpackOptions) => {
  try {
    // match webpack.config.js and all its weird friends
    invariant(
      path.basename(module.parent.filename).startsWith('webpack.config.'),
      '`wub` should be required from a webpack config file'
    );

    checkDeps();

    const normalisedPresets = normalisePresets(
      // prepend default preset
      [require('./defaultPreset')].concat(presets)
    );

    let mergedConfig;

    // loop through preset functions
    for (let idx = -1, len = normalisedPresets.length; ++idx < len; ) {
      const preset = normalisedPresets[idx];

      const presetConfig = preset(env, webpackOptions);

      mergedConfig = webpackMerge.smartStrategy({ entry: 'replace' })(
        mergedConfig,
        presetConfig
      );
    }

    return mergedConfig;
  } catch (e) {
    console.error(
      styleStart('red', 'bold') +
        '[WUB] Error: ' +
        e.message +
        styleEnd() +
        '\n\n' +
        e.stack
    );
    process.exit(69); // nice
  }
};
