const invariant = require('invariant');

const webpackOptionsSchema = require.main.require(
  'webpack/schemas/webpackOptionsSchema'
);

const {
  validateSchema: validateWebpackSchema,
  WebpackOptionsValidationError,
} = require.main.require('webpack');

function validateConfig(config) {
  const errors = validateWebpackSchema(webpackOptionsSchema, config).filter(
    // ignore missing entry errors
    f => f.params === null || f.params.missingProperty !== 'entry'
  );
  if (errors.length) {
    throw new WebpackOptionsValidationError(errors);
  }
  return config;
}

/**
 * @param {object} presets preset arrays passed to `wub` merged into one
 * @returns {array} array of functions that return webpack configs
 */
function normalisePresets(presets) {
  // expand presets
  const normalisedPresets = presets.map((preset, idx) => {
    const pType = typeof preset;

    if (pType === 'function') {
      // validate output of preset function
      return (...args) => validateConfig(preset(...args));
    } else if (pType === 'object') {
      // convert raw webpack config into a function that returns a config
      return () => validateConfig(preset);
    } else {
      invariant(
        false,
        'Invalid preset at `presets[%s]`. Only function and object types are allowed (received `%s`).',
        idx,
        pType
      );
    }
  });

  return normalisedPresets;
}

module.exports = normalisePresets;
